@@ -18,7 +18,7 @@
 #'     \item{\code{psi0}:}{Object of class \code{"matrix"}: a matrix with named columns containing the initial estimates for the parameters in the model (first line) and for the covariate effects (second and subsequent lines, optional). The number of columns should be equal to the number of parameters in the model.}
 #'     \item{\code{transform.par}:}{Object of class \code{"numeric"}: vector giving the distribution for each model parameter (0: normal, 1: log-normal, 2: logit, 3: probit). Its length should be equal to the number of parameters in the model.}
 #'     \item{\code{fixed.estim}:}{Object of class \code{"numeric"}: for each parameter, 0 if the parameter is fixed and 1 if it should be estimated. Defaults to a vector of 1 (all parameters are estimated). Its length should be equal to the number of parameters in the model.}
-#'     \item{\code{error.model}:}{Object of class \code{"character"}: name of the error model. Valid choices are "constant" (default), "proportional" and "combined" (see equations in User Guide)}
+#'     \item{\code{error.model}:}{Object of class \code{"character"}: name of the error model. Valid choices are "constant" (default), "proportional" and "combined" (see equations in User Guide, except for combined which was changed to y = f + sqrt(a^2+b^2*f^2)*e )}
 #'     \item{\code{covariate.model}:}{Object of class \code{"matrix"}: a matrix of 0's and 1's, with a 1 indicating that a parameter-covariate relationship is included in the model (and an associated fixed effect will be estimated). The nmuber of columns should be equal to the number of parameters in the model and the number of rows to the number of covariates.}
 #'     \item{\code{covariance.model}:}{Object of class \code{"matrix"}: a matrix f 0's and 1's giving the structure of the variance-covariance matrix. Defaults to the Identity matrix (diagonal IIV, no correlations between parameters)}
 #'     \item{\code{omega.init}:}{Object of class \code{"matrix"}: a matrix giving the initial estimate for the variance-covariance matrix}
@@ -35,12 +35,15 @@
 #'     \item{showall}{\code{signature(object = "SaemixModel")}: shows all the elements in the object}
 #'     \item{show}{\code{signature(object = "SaemixModel")}: prints details about the object}
 #' 	 }
-#' @references Comets  E, Lavenu A, Lavielle M. Parameter estimation in nonlinear mixed effect models using saemix, an R implementation of the SAEM algorithm. Journal of Statistical Software 80, 3 (2017), 1-41.
+#' @references E Comets, A Lavenu, M Lavielle M (2017). Parameter estimation in nonlinear mixed effect models using saemix,
+#' an R implementation of the SAEM algorithm. Journal of Statistical Software, 80(3):1-41.
 #' 
-#' Kuhn E, Lavielle M. Maximum likelihood estimation in nonlinear mixed effects models. Computational Statistics and Data Analysis 49, 4 (2005), 1020-1038.
+#' E Kuhn, M Lavielle (2005). Maximum likelihood estimation in nonlinear mixed effects models. 
+#' Computational Statistics and Data Analysis, 49(4):1020-1038.
+#' 
+#' E Comets, A Lavenu, M Lavielle (2011). SAEMIX, an R version of the SAEM algorithm. 20th meeting of the 
+#' Population Approach Group in Europe, Athens, Greece, Abstr 2173.
 #' 
-#' Comets E, Lavenu A, Lavielle M. SAEMIX, an R version of the SAEM algorithm. 20th meeting of the 
-#' Population Approach Group in Europe, Athens, Greece (2011), Abstr 2173.
 #' @author Emmanuelle Comets \email{emmanuelle.comets@@inserm.fr}
 #' @author Audrey Lavenu
 #' @author Marc Lavielle.
@@ -58,6 +61,7 @@ setClass(
   representation=representation(
     model="function", 		# name of model function
     description="character",	# model description
+    modeltype="character",     # type of model (structural, for continuous responses, or likelihood)
     psi0="matrix",		# CI for parameter estimates
     transform.par="numeric",	# distribution for model parameters
     fixed.estim="numeric",	# 1 for fixed parameters estimated
@@ -100,10 +104,12 @@ setClass(
       message("[ SaemixModel : Error ] The number of parameters should be the same in the following elements: psi0 (initial conditions), transform.par, fixed.estim, covariate.model, and the matrices covariance.model and omega.init should be square matrices of size equal to the number of parameters. Please check the input.")
       return("Size mismatch")
     }
-    if(npar<2) {
-      message("[ SaemixModel : Error ] SAEM needs at least two parameters to work on.")
-      return("Psi0 has size 1")
-    }
+    
+    # if(npar<2) {
+    #   message("[ SaemixModel : Error ] SAEM needs at least two parameters to work on.")
+    #   return("Psi0 has size 1")
+    # }
+
 		if(sum(object@fixed.estim*mydiag(object@covariance.model))==0) {
 			message("[ SaemixModel : Error ] ")
 			if(sum(mydiag(object@covariance.model))==0) message("At least one parameter with IIV must be included in the model.") else message("At least one parameter with IIV must be estimated and not fixed in the model.")
@@ -113,6 +119,10 @@ setClass(
       message("[ SaemixModel : Error ] Invalid residual error model")
       return("Invalid residual error model")
     }
+    if(is.na(match(object@modeltype,c("structural","likelihood")))) {
+      message("[ SaemixModel : Error ] Invalid type of model")
+      return("Invalid model type")
+    }
     return(TRUE)
   }
 )
@@ -126,6 +136,7 @@ setClass(
 #' below).
 #' @param description a character string, giving a brief description of the
 #' model or the analysis
+#' @param modeltype a character string, giving the type of the model for the analysis (one of "structural" or "likelihood", defaults to structural)
 #' @param psi0 a matrix with a number of columns equal to the number of
 #' parameters in the model, and one (when no covariates are available) or two
 #' (when covariates enter the model) giving the initial estimates for the fixed
@@ -149,7 +160,11 @@ setClass(
 #' or covariances (off-diagonal elements). Defaults to the identity matrix
 #' @param omega.init a square matrix of size equal to the number of parameters
 #' in the model, giving the initial estimate for the variance-covariance matrix
-#' of the model. Defaults to the identity matrix
+#' of the model. The current default is a diagonal matrix with ones for all
+#' transformed parameters as well as for all untransformed parameters with an
+#' absolute value smaller than one.  For untransformed parameters greater or
+#' equal to one, their squared value is used as the corresponding diagonal
+#' element.
 #' @param error.init a vector of size 2 giving the initial value of a and b in
 #' the error model. Defaults to 1 for each estimated parameter in the error model
 #' @param name.modpar names of the model parameters, if they are not given as
@@ -160,8 +175,10 @@ setClass(
 setMethod(
   f="initialize",
   signature="SaemixModel",
-  definition=function(.Object,model,description,psi0, name.response, name.sigma, transform.par,fixed.estim, error.model,covariate.model,covariance.model,omega.init,error.init, name.modpar, verbose=TRUE){
+  definition=function(.Object,model,description,modeltype,psi0, name.response, name.sigma, transform.par,fixed.estim, error.model,covariate.model,covariance.model,omega.init,error.init, name.modpar, verbose=TRUE){
 #    cat ("--- initialising SaemixModel Object --- \n")
+    if(missing(modeltype)) modeltype<-"structural"
+    .Object@modeltype<-modeltype
     if(missing(model)) {
 #      cat("Error initialising SaemixModel object:\n   The model must be a function, accepting 3 arguments: psi (a vector of parameters), id (a vector of indices) and xidep (a matrix of predictors). Please see the documentation for examples.\n")
       return(.Object)
@@ -176,14 +193,14 @@ setMethod(
     npar<-dim(psi0)[2]
     if(missing(name.modpar) || length(name.modpar)==0) {
       y1<-try(name.modpar<-colnames(psi0))
-      if(class(y1)=="try-error") {
+      if(inherits(y1,"try-error")) {
         if(verbose) message("     Can't find parameter names.\n")
         name.modpar<-paste("theta",1:npar)
       }
     }
     if(is.null(colnames(psi0))) {
       y1<-try(colnames(psi0)<-name.modpar)
-      if(class(y1)=="try-error") {
+      if(inherits(y1,"try-error")) {
         if(verbose) message("Warning:\n   Problem with names of psi0\n")
         colnames(psi0)<-name.modpar<-paste("theta",1:npar)
       }
@@ -232,6 +249,7 @@ setMethod(
       if(verbose) message("Error initialising SaemixModel object:\n   The covariance model needs to have the same size as the number of parameters.\n")
       return(.Object)
     }
+    if(!validate.covariance.model(covariance.model, verbose)) return(.Object)
     if(is.null(colnames(covariance.model))) colnames(covariance.model)<-rownames(covariance.model)<-colnames(psi0)
     .Object@covariance.model<-covariance.model
     indx.omega<-which(diag(covariance.model)>0)
@@ -267,7 +285,7 @@ setMethod(
 # error models :
 #   constant            y = f + a*e
 #   proportional        y = f + b*f*e
-#   combined            y = f + (a+b*f)*e
+#   combined            y = f + sqrt(a^2+b^2*f^2)*e
 #   exponential         y = f*exp(a*e)    ( <=>  log(y) = log(f) + a*e )
     if(missing(error.init) || length(error.init)!=2*length(.Object@error.model)) {
       error.init<-c()
@@ -283,7 +301,9 @@ setMethod(
     if(missing(name.sigma)) mis.sig<-TRUE else mis.sig<-FALSE
     if(missing(name.sigma) || length(name.sigma)!=2) name.sigma<-c("a","b")
     if(!mis.sig) { # & .Object@name.response!="" # pb if response has more than 1 element
-      for(i in 1:length(.Object@name.response)) xres<-c(xres,paste(name.sigma,.Object@name.response[i],sep="."))
+      for(i in 1:length(.Object@name.response)) {
+        if(.Object@name.response[i]!="") xres<-c(xres,paste(name.sigma,.Object@name.response[i],sep=".")) else xres<-c(xres,paste(name.sigma,i,sep=".")) # xres<-c(xres,name.sigma) ?
+      }
     } else xres<-rep(name.sigma,length(.Object@name.response))
     .Object@name.sigma<-xres
     .Object@error.init<-error.init
@@ -328,7 +348,7 @@ setMethod(
 
 #' Get/set methods for SaemixModel object
 #' 
-#' Access slots of a SaemixModel object using the object["slot"] format
+#' Access slots of a SaemixModel object using the object\["slot"\] format
 #' 
 #' @param x object
 #' @param i element to be replaced
@@ -347,6 +367,7 @@ setMethod(
   switch (EXPR=i,
     "model"={return(x@model)},
     "description"={return(x@description)},
+    "modeltype"={return(x@modeltype)},
     "psi0"={return(x@psi0)},
     "transform.par"={return(x@transform.par)},
     "fixed.estim"={return(x@fixed.estim)},
@@ -383,6 +404,7 @@ setReplaceMethod(
   switch (EXPR=i,
     "model"={x@model<-value},
     "description"={return(x@description)},
+    "modeltype"={return(x@modeltype)},
     "psi0"={x@psi0<-value},
     "transform.par"={x@transform.par<-value},
     "fixed.estim"={x@fixed.estim<-value},
@@ -423,10 +445,17 @@ setReplaceMethod(
 setMethod("print","SaemixModel",
   function(x,...) {
     cat("Nonlinear mixed-effects model\n")
+    if( is.null(body(x@model))) {
+      cat("No model function set yet\n")
+      return()
+    }
     distrib<-c("normal","log-normal","probit","logit")
     cat("  Model function")
     if(length(x@description)>0 && nchar(x@description)>0) cat(": ",x@description)
     cat("\n")
+    cat("  Model type")
+    if(length(x@modeltype)>0 && nchar(x@modeltype)>0) cat(": ",x@modeltype)
+    cat("\n")
     print(x@model)
     cat("  Nb of parameters:",x@nb.parameters,"\n")
     cat("      parameter names: ",x@name.modpar,"\n")
@@ -438,7 +467,9 @@ setMethod("print","SaemixModel",
 #    try(colnames(tab)<-rownames(tab)<-x@name.modpar)
     print(tab,quote=FALSE)
     st1<-paste(x@name.sigma,x@error.init,sep="=")
+    if (x@modeltype=="structural"){
     cat("  Error model:",x@error.model,", initial values:",st1[x@indx.res],"\n")
+    }
    if(dim(x@covariate.model)[1]>0) {
       cat("  Covariate model:")
       if(sum(x@covariate.model)==0) cat(" none\n") else {
@@ -466,7 +497,8 @@ setMethod("show","SaemixModel",
     }
     fix1<-ifelse(object@fixed.estim==1,""," [fixed]")
     cat("    ",object@nb.parameters,"parameters:", paste(object@name.modpar,fix1,sep=""),"\n")
-    cat("     error model:",object@error.model,"\n")
+    if (object@modeltype=="structural")
+      cat("     error model:",object@error.model,"\n")
     if(dim(object@covariate.model)[1]>0) {
       cat("     covariate model:\n")
       print(object@covariate.model) 
@@ -550,7 +582,7 @@ setMethod("summary","SaemixModel",
     distrib<-c("normal","log-normal","probit","logit")
     tab.par<-data.frame(Parameter=object@name.modpar, Distribution=distrib[object@transform.par+1], Estimated=ifelse(as.numeric(object@betaest.model[1,])==1,"estimated","fixed"), Initial.value=object@psi0[1,])
     tab.res<-data.frame(parameters=object@name.sigma,Initial.value=object@error.init)   
-    res<-list(model=list(model.function=object@model, error.model=object@error.model),parameters=list(fixed=tab.par, residual.error=tab.res),covariance.model=object@covariance.model, covariate.model=object@covariate.model)
+    res<-list(model=list(modeltype=object@modeltype,model.function=object@model, error.model=object@error.model),parameters=list(fixed=tab.par, residual.error=tab.res),covariance.model=object@covariance.model, covariate.model=object@covariate.model)
     invisible(res)
  }
 )
@@ -563,40 +595,73 @@ setMethod("summary","SaemixModel",
 #' 
 #' This function will plot predictions obtained from an SaemixModel object over a given range of X. Additional predictors may be passed on to the function using the predictors argument.
 #' 
+## #' @name plot-SaemixModel
+#' 
 #' @param x an SaemixData object or an SaemixSimData object
 #' @param y unused, present for compatibility with base plot function
-#' @param range range of X over which the model is to be plotted
+#' @param range range of X over which the model is to be plotted. Important note: the *first* predictor will be used for the X-axis, the other
+#' predictors when present need to be passed sequentially in the predictors argument, in the order in which they appear in the model
+#' Less important note: please use explicitely range=XXX where XXX is of the form c(a,b) to pass the plotting range on the X-axis)
 #' @param psi parameters of the model 
 #' @param predictors additional predictors needed to pass on to the model
-#' @param ... additional arguments to be passed on to plot (titles, legends, ...)
+#' @param ... additional arguments to be passed on to plot (titles, legends, ...). Use verbose=TRUE to print some messages 
+#' concerning the characteristics of the plot
 #' 
 #' @aliases plot,SaemixModel-methods 
 #' @aliases plot,SaemixModel
+#' @aliases plot-SaemixModel
 #' @keywords methods
 ### #' @docType methods
 #' @exportMethod plot
 #' @rdname plot-SaemixModel
+#' 
+#'@examples
+#' # Note that we have written the PK model so that time is the first predictor (xidep[,1]) 
+#' # and dose the second
+#' model1cpt<-function(psi,id,xidep) { 
+#'      tim<-xidep[,1]  
+#'      dose<-xidep[,2]
+#'      ka<-psi[id,1]
+#'      V<-psi[id,2]
+#'      CL<-psi[id,3]
+#'      k<-CL/V
+#'      ypred<-dose*ka/(V*(ka-k))*(exp(-k*tim)-exp(-ka*tim))
+#'      return(ypred)
+#'      }
+#'  x<-saemixModel(model=model1cpt,description="One-compartment model with first-order absorption", 
+#'                psi0=matrix(c(1.5,30,1), ncol=3,byrow=TRUE, dimnames=list(NULL, c("ka","V","CL"))))
+#'  # Plot the model over 0-24h, using the parameters given in psi0 and a dose of 300
+#'  plot(x, range=c(0,24), predictors=300, verbose=TRUE)
+#'  # Plot the model over 0-24h, using another set of parameters and a dose of 350
+#'  plot(x, range=c(0,24), psi=c(1.5,20,2), predictors=350, verbose=TRUE)
+
 
 # Plot simulations from the model
 # ECO TODO: adjust to multiple responses
 
 setMethod("plot","SaemixModel",
-  function(x,y,range=c(0,1),psi,predictors,...) {
-    if(missing(psi)) psi<-x@psi0[1,]
-    psi<-matrix(psi,nrow=1)
+  function(x, y , range=c(0,1), psi=NULL, predictors=NULL, ...) {
+    # If verbose=TRUE, print messages
+    args1<-match.call(expand.dots=TRUE)
+    list.args <- list(...)
+    i1<-match("verbose",names(args1))
+    if(!is.na(i1)) verbose<-FALSE else verbose<-eval(args1[[i1]])
+    # Set psi by default to the starting parameters given in the model (if not given as arguments)
+    if(is.null(psi)) psi<-x@psi0[1,,drop=FALSE]
+    if(is.null(dim(psi)[1])) psi<-matrix(psi,nrow=1)
     npred<-length(x@name.predictors)
-    if(npred==0 & missing(predictors)) npred<-1 else {
+    if(npred==0 & is.null(predictors)) npred<-1 else {
       if(npred==0 & !missing(predictors)) {
         npred<-1+length(predictors)
       } else {
         if(npred>1 & (missing(predictors) || length(predictors)<(npred-1))) {
-        message("Please provide the value of the predictors other than X\n")
+        if(verbose) message("Please provide the value of the predictors other than X\n")
         return("Missing predictors")
       }
      }
     }
     if(length(x@name.response)>1) {
-      message("Currently the plot can only be obtained for single-response models.\n")
+      if(verbose) message("Currently the plot can only be obtained for single-response models.\n")
       return()
     }
     npts<-100
@@ -613,9 +678,9 @@ setMethod("plot","SaemixModel",
       } else colnames(xdep)<-paste("Predictor",1:npred)
     }
     ypred<-try(x@model(psi,id,xdep))
-    if(!is.numeric(ypred)) {
+    if(!is.numeric(ypred) & verbose) {
       message("Problem when attempting to obtain predictions from the model.\n")
-      message("Usage: plot(x,range=c(0,1),psi,predictors) \n")
+      message("Usage: plot(x, range=c(0,1), psi, predictors) \n")
       message("Possible solutions can be:\n")
       message("   1. provide suitable values for X (option range=c(<lower bound>, <upper bound>))\n")
       message("   2. provide values for additional predictors (option predictors=c(<value for predictor 1>, <value for predictor 2>, ...)).\n")
@@ -623,7 +688,7 @@ setMethod("plot","SaemixModel",
       message("   4. the predictor used the X-axis is assumed to be in the first column; please check your model is written in a compatible way.\n")
     } else {
       if(length(x@name.X)==0 | length(x@name.predictors)==0) message("Warning: X predictor supposed to be on the first axis\n")
-      message("Plot characteristics:\n")
+      if(verbose) message("Plot characteristics:\n")
       if(npred>1) {
         for(j in 1:dim(xdep)[2]) {
     if(length(x@name.X)==0) {
@@ -632,13 +697,238 @@ setMethod("plot","SaemixModel",
       if(colnames(xdep)[j]!=x@name.X) message("    predictor:",colnames(xdep)[j],"=",xdep[1,j],"\n")
     }
       }}
-      message("   range for X-axis:",min(xval),"-",max(xval),"\n")
-      message("   parameters used in the simulation:", paste(x@name.modpar,"=",psi[1,],collapse=", "),"\n")
+      if(verbose) message("   range for X-axis:",min(xval),"-",max(xval),"\n")
+      if(verbose) message("   parameters used: ", paste(x@name.modpar,"=",psi[1,],collapse=", "),"\n")
       plot(xval,ypred,type="l",xlab=ifelse(length(x@name.X)==0, "X",x@name.X),ylab=ifelse(length(x@name.response)==0, "Response",x@name.response))
     }
   }
 )
 
+####################################################################################
+####			SaemixModel class - method to obtain predictions given a set of predictors and parameters			####
+####################################################################################
+
+#' Predictions for a new dataset
+#' 
+#' @param object an SaemixModel object
+#' @param psi a vector or a dataframe giving the parameters for which predictions are to be computed (defaults to empty). 
+#' The number of columns in psi (or the number of elements of psi, if psi is given as a vector) should match the number of
+#' parameters in the model, otherwise an error message will be shown and the function will return empty.
+#' If psi is NA, the predictions are computed for the population parameters in the model (first line of the psi0 slot). 
+#' If psi is a dataframe, each line will be used for a separate 'subject' in the predictors dataframe, as 
+#' indicated by the id argument; if id is not given, only the first line of psi will be used. 
+#' @param predictors a dataframe with the predictors for the model (must correspond to the predictors used by the model function)
+#' @param id a vector of indices of length equal to the number of lines in predictors, matching each line of predictors to the 
+#' corresponding line in psi, ie the parameters for this predictors (defaults to empty). If id is given, the unique values in id must be equal
+#' to the number of lines in psi, otherwise id will be set to 1. If id is given and its values do not take the consecutive values 1:N, the
+#' indices will be matched to 1:N to follow the lines in psi.
+#' @param \dots unused argument, for consistency with the generic
+#' 
+#' @details The function uses the model slot of the SaemixModel object to obtain predictions, using the predictors object. The
+#' user is responsible for giving all the predictors needed by the model function.
+#' if psi is not given, the predictions will be computed for the population parameters (first line of the psi0 slot) of the object.
+#' 
+#' @details The predictions correspond to the structure of the model; for models defined in terms of their likelihood, the predictions 
+#' are the log-pdf of the model (see documentation for details).
+#' 
+#' @details Warning: this function is currently under development and the output may change in future versions of the package 
+#' to conform to the usual predict functions.
+#' 
+#' @return a list with two components
+#' \describe{
+#' \item{param}{a dataframe with the estimated parameters}
+#' \item{predictions}{a dataframe with the population predictions}
+#' }
+#' 
+#' @examples 
+#' data(theo.saemix)
+#' xpred<-theo.saemix[,c("Dose","Time")]
+#' 
+#' model1cpt<-function(psi,id,xidep) { 
+#' 	  dose<-xidep[,1]
+#' 	  tim<-xidep[,2]  
+#' 	  ka<-psi[id,1]
+#' 	  V<-psi[id,2]
+#' 	  CL<-psi[id,3]
+#' 	  k<-CL/V
+#' 	  ypred<-dose*ka/(V*(ka-k))*(exp(-k*tim)-exp(-ka*tim))
+#' 	  return(ypred)
+#' }
+#' 
+#' saemix.model<-saemixModel(model=model1cpt,modeltype="structural",
+#'   description="One-compartment model with first-order absorption", 
+#'   psi0=matrix(c(1.,20,0.5,0.1,0,-0.01),ncol=3, byrow=TRUE,
+#'   dimnames=list(NULL, c("ka","V","CL"))),transform.par=c(1,1,1),
+#'   covariate.model=matrix(c(0,1,0,0,0,0),ncol=3,byrow=TRUE),fixed.estim=c(1,1,1),
+#'   covariance.model=matrix(c(1,0,0,0,1,0,0,0,1),ncol=3,byrow=TRUE),
+#'   omega.init=matrix(c(1,0,0,0,1,0,0,0,1),ncol=3,byrow=TRUE),error.model="constant")
+#' 
+#' head(predict(saemix.model, xpred)$predictions)
+#' head(predict(saemix.model, xpred, psi=c(2, 40, 0.5))$predictions)
+#' indpsi<-data.frame(ka=2, V=seq(25,47,2), CL=seq(2.5,4.7, 0.2))
+#' head(predict(saemix.model, xpred, psi=indpsi)$predictions)
+#' 
+#' @importFrom stats predict
+#' @method predict SaemixModel
+#' @export 
+#' 
+
+predict.SaemixModel<-function(object, predictors, psi=c(), id=c(), ...) {
+  xidep<-predictors
+  if(length(id)==0 || length(id)!=dim(predictors)[1]) 
+    id<-rep(1,dim(xidep)[1]) 
+  idkeep<-id
+  if(max(id)>length(unique(id))) { # indexes need to go from 1 to N
+    id1<-1:length(unique(id))
+    id2<-unique(id)
+    id<-id1[match(id,id2)]
+  }
+  if(length(psi)==0) psi<-object["psi0"][1,,drop=FALSE]
+  if(is.null(dim(psi))) psi<-as.data.frame(t(psi)) # psi given as a vector
+  if(dim(psi)[2] != object@nb.parameters) {
+    message(paste0("psi must have a number of columns equal to the number of parameters in the model (",object@nb.parameters,")\n")
+    )
+    return()
+  }
+  if(dim(psi)[1]==1 & length(unique(id))>1)
+    psi<-do.call(rbind,rep(list(psi),length(unique(id))))
+  colnames(psi)<-colnames(object["psi0"])
+  rownames(psi)<-NULL
+  ypred<-object["model"](psi, id, xidep)
+  return(list(param=cbind(id=1:dim(psi)[1],psi), predictions=data.frame(id=idkeep, xidep, pred=unname(ypred))))
+}
+
+
+####################################################################################
+####			SaemixModel & SaemixData class - method to plot	predictions from a model for the data in a dataset		####
+####################################################################################
+
+#' Plot model predictions for a new dataset. If the dataset is large, only the first 20 subjects (id's) will be shown.
+#' 
+#' @param x an SaemixModel object
+#' @param y an SaemixData object
+#' @param ... additional arguments. Passing psi=X where X is a vector or a dataframe will allow
+#' changing the parameters for which predictions are to be computed (defaults to the population parameters
+#' defined by the psi element of x) (see details)
+#' 
+#' @details The function uses the model slot of the SaemixModel object to obtain predictions, using the dataset contained in the 
+#' SaemixData object. The user is responsible for making sure data and model match.
+#' If psi is not given, the predictions will be computed for the population parameters (first line of the psi0 slot) of the object.
+#' If psi is given, the number of columns in psi (or the number of elements of psi, if psi is given as a vector) should match 
+#' the number of parameters in the model, otherwise an error message will be shown and the function will return empty.
+#' If psi is a dataframe, each line will be used for a separate subject of the smx.data object. Elements of psi will be recycled 
+#' if psi has less lines than the number of subjects in the dataset.
+#' 
+#' @details Currently this function only works for models defined as 'structural'.
+#' 
+#' @return a ggplot object
+#' 
+#' @aliases plot.SaemixModel
+#' 
+#' @examples 
+#' data(theo.saemix)
+#' saemix.data<-saemixData(name.data=theo.saemix,header=TRUE,sep=" ",na=NA,
+#'    name.group=c("Id"),name.predictors=c("Dose","Time"),
+#'    name.response=c("Concentration"),name.covariates=c("Weight","Sex"),
+#'    units=list(x="hr",y="mg/L", covariates=c("kg","-")), name.X="Time")
+#' 
+#' model1cpt<-function(psi,id,xidep) { 
+#' 	  dose<-xidep[,1]
+#' 	  tim<-xidep[,2]  
+#' 	  ka<-psi[id,1]
+#' 	  V<-psi[id,2]
+#' 	  CL<-psi[id,3]
+#' 	  k<-CL/V
+#' 	  ypred<-dose*ka/(V*(ka-k))*(exp(-k*tim)-exp(-ka*tim))
+#' 	  return(ypred)
+#' }
+#' 
+#' saemix.model<-saemixModel(model=model1cpt,modeltype="structural",
+#'   description="One-compartment model with first-order absorption", 
+#'   psi0=matrix(c(1.,20,0.5,0.1,0,-0.01),ncol=3, byrow=TRUE,
+#'   dimnames=list(NULL, c("ka","V","CL"))),transform.par=c(1,1,1),
+#'   covariate.model=matrix(c(0,1,0,0,0,0),ncol=3,byrow=TRUE),fixed.estim=c(1,1,1),
+#'   covariance.model=matrix(c(1,0,0,0,1,0,0,0,1),ncol=3,byrow=TRUE),
+#'   omega.init=matrix(c(1,0,0,0,1,0,0,0,1),ncol=3,byrow=TRUE),error.model="constant")
+#' 
+#' plot(saemix.model, saemix.data)
+#' plot(saemix.model, saemix.data, psi=c(2, 40, 3))
+#' indpsi<-data.frame(ka=2, V=seq(25,47,2), CL=seq(2.5,4.7, 0.2))
+#' plot(saemix.model, saemix.data, psi=indpsi)
+#' 
+#' @exportMethod plot
+#' 
+
+# Plot the data, either as points or as lines grouped by x@name.group
+setMethod("plot",c("SaemixModel","SaemixData"),
+          function(x, y, ...) {
+            if(x@modeltype!="structural") {
+              message("Currently plots of the model are only available for continuous response models\n")
+              return()
+            }
+            args1<-match.call(expand.dots=TRUE)
+            list.args <- list(...)
+            i1<-match("verbose",names(args1))
+            if(!is.na(i1)) verbose<-FALSE else verbose<-eval(args1[[i1]])
+            i1<-match("psi",names(args1))
+            if(!is.na(i1)) psi<-eval(args1[[i1]]) else psi<-x["psi0"][1,,drop=FALSE]
+            if(is.null(dim(psi))) psi<-as.data.frame(t(psi)) # psi given as a vector
+            if(dim(psi)[2] != x@nb.parameters) {
+              message(paste0("psi must have a number of columns equal to the number of parameters in the model (",x@nb.parameters,")\n"))
+              return()
+            }
+            if(dim(psi)[1]==1 || dim(psi)[1]<y@N)
+              psi<-do.call(rbind,rep(list(psi),length.out=y@N))
+            i1<-match("ilist",names(args1))
+            if(!is.na(i1)) ilist<-eval(args1[[i1]]) else ilist<-NA
+            
+            nvalues<-100
+            xt<-seq(min(y@data[,y@name.X]), max(y@data[,y@name.X]), length.out=nvalues)
+            xidep<-data.frame(x=xt)
+            colnames(xidep)<-y@name.X
+            id<-y@data[,y@name.group]
+            if(length(y@name.predictors)>1) {
+              otherpred<-y@name.predictors[y@name.predictors != y@name.X]
+              x1<-y@data[match(unique(id), id), otherpred, drop=FALSE]
+              dat1<-NULL
+              for(i in 1:length(unique(id)))
+                dat1<-rbind(dat1, 
+                            do.call(rbind,rep(list(x1[i,,drop=FALSE]), nvalues)))
+              xidep<-cbind(xidep, dat1)
+              colnames(xidep[2:dim(xidep)[2]])<-otherpred
+              xidep<-xidep[,y["name.predictors"]] # Sort the predictors back in the correct order...
+            }
+            id<-rep(1:length(unique(id)), each=nvalues)
+            ypred<-predict(x, predictors=xidep, psi=psi, id=id)
+            gpred<-cbind(id=id,xidep,y=ypred$predictions$pred)
+            colnames(gpred)[colnames(gpred)==y@name.X]<-"x"
+            
+            gdat<-y@data
+            colnames(gdat)[colnames(gdat)==y@name.X]<-"x"
+            colnames(gdat)[colnames(gdat)==y@name.response]<-"y"
+            colnames(gdat)[colnames(gdat)==y@name.group]<-"id"
+            zesuj<-unique(gdat$id)
+            if(!is.na(ilist)[1]) {
+              if(is(ilist,"numeric")) ilist<-zesuj[intersect(ilist, 1:length(zesuj))] else ilist<-intersect(ilist, zesuj)
+            } else ilist<-zesuj[1:min(20, length(zesuj))]
+            gdat1<-gdat[gdat$id %in% ilist,]
+            gpred1<-gpred[gpred$id %in% ilist,]
+            if(length(unique(gdat$id))>20) {
+              nrow<-4
+              ncol<-5
+            } else {
+              nrow<-NULL
+              ncol<-NULL
+            }
+            
+            g1<-ggplot(data=gdat1, aes(x=.data$x, y=.data$y, group=.data$id)) + geom_point() + geom_line(data=gpred1,aes(x=.data$x, y=.data$y)) +
+              facet_wrap(.~id, nrow=nrow, ncol=ncol) + 
+              labs(x=y@name.X, y=y@name.response) + theme_bw()
+            return(g1)
+          } 
+)
+
+
 ####################################################################################
 ####			SaemixModel class - User-level function			####
 ####################################################################################
@@ -668,6 +958,7 @@ setMethod("plot","SaemixModel",
 #' named vector.
 #' @param description a character string, giving a brief description of the
 #' model or the analysis
+#' @param modeltype a character string, giving model type (structural or likelihood)
 #' @param name.response the name of the dependent variable
 #' @param name.sigma a vector of character string giving the names of the residual error parameters
 #' @param error.model type of residual error model (valid types are constant,
@@ -685,7 +976,7 @@ setMethod("plot","SaemixModel",
 #' (off-diagonal elements). Defaults to the identity matrix
 #' @param omega.init a square matrix of size equal to the number of parameters
 #' in the model, giving the initial estimate for the variance-covariance matrix
-#' of the model. Defaults to the identity matrix
+#' of the model.
 #' @param error.init a vector of size 2 giving the initial value of a and b in
 #' the error model. Defaults to 1 for each estimated parameter in the error
 #' model
@@ -697,13 +988,15 @@ setMethod("plot","SaemixModel",
 #' Marc Lavielle.
 #' @seealso \code{\link{SaemixData}},\code{\link{SaemixModel}},
 #' \code{\link{saemixControl}},\code{\link{saemix}}
-#' @references Comets  E, Lavenu A, Lavielle M. Parameter estimation in nonlinear mixed effect models using saemix, an R implementation of the SAEM algorithm. Journal of Statistical Software 80, 3 (2017), 1-41.
+#' @references E Comets, A Lavenu, M Lavielle M (2017). Parameter estimation in nonlinear mixed effect models using saemix,
+#' an R implementation of the SAEM algorithm. Journal of Statistical Software, 80(3):1-41.
 #' 
-#' Kuhn E, Lavielle M. Maximum likelihood estimation in nonlinear mixed effects models. Computational Statistics and Data Analysis 49, 4 (2005), 1020-1038.
+#' E Kuhn, M Lavielle (2005). Maximum likelihood estimation in nonlinear mixed effects models. 
+#' Computational Statistics and Data Analysis, 49(4):1020-1038.
+#' 
+#' E Comets, A Lavenu, M Lavielle (2011). SAEMIX, an R version of the SAEM algorithm. 20th meeting of the 
+#' Population Approach Group in Europe, Athens, Greece, Abstr 2173.
 #' 
-#' Comets E, Lavenu A, Lavielle M. SAEMIX, an R version of the SAEM algorithm.
-#' 20th meeting of the Population Approach Group in Europe, Athens, Greece
-#' (2011), Abstr 2173.
 #' @keywords models
 #' @examples
 #' 
@@ -728,14 +1021,14 @@ setMethod("plot","SaemixModel",
 #' 
 #' @export saemixModel
 
-saemixModel<-function(model,psi0,description="", name.response="", name.sigma=character(), error.model=character(), transform.par=numeric(),fixed.estim=numeric(),covariate.model=matrix(nrow=0,ncol=0), covariance.model=matrix(nrow=0,ncol=0),omega.init=matrix(nrow=0,ncol=0),error.init=numeric(), name.modpar=character(), verbose=TRUE) {
+saemixModel<-function(model,psi0,description="",modeltype ="structural", name.response="", name.sigma=character(), error.model=character(), transform.par=numeric(),fixed.estim=numeric(),covariate.model=matrix(nrow=0,ncol=0), covariance.model=matrix(nrow=0,ncol=0),omega.init=matrix(nrow=0,ncol=0),error.init=numeric(), name.modpar=character(), verbose=TRUE) {
 # Creating model from class
   if(missing(model)) {
     if(verbose) cat("Error in saemixModel:\n   The model must be a function, accepting 3 arguments: psi (a vector of parameters), id (a vector of indices) and xidep (a matrix of predictors). Please see the documentation for examples.\n")
     return("Creation of SaemixModel failed")  
   }
   xcal<-try(typeof(model))
-  if(class(xcal)=="try-error") {
+  if(inherits(xcal,"try-error")) {
     if(verbose) cat("Error in saemixModel:\n   the model function does not exist.\n")
     return("Creation of SaemixModel failed")  
   }
@@ -766,9 +1059,9 @@ saemixModel<-function(model,psi0,description="", name.response="", name.sigma=ch
   if(is.null(colnames(psi0))) {
     if(verbose) cat("Warning: no names given for the parameters in the model, please consider including parameter names.\n")
   }
-  xmod<-try(new(Class="SaemixModel",model=model,description=description ,psi0=psi0, name.response=name.response, name.sigma=name.sigma, error.model=error.model, transform.par=transform.par,fixed.estim=fixed.estim, covariate.model=covariate.model,covariance.model=covariance.model, omega.init=omega.init,error.init=error.init,name.modpar=name.modpar))
+  xmod<-try(new(Class="SaemixModel",model=model,description=description , modeltype=modeltype,psi0=psi0, name.response=name.response, name.sigma=name.sigma, error.model=error.model, transform.par=transform.par,fixed.estim=fixed.estim, covariate.model=covariate.model,covariance.model=covariance.model, omega.init=omega.init,error.init=error.init,name.modpar=name.modpar))
   if(class(xmod)=="SaemixModel") x1<-try(validObject(xmod),silent=FALSE) else x1<-xmod
-  if(class(x1)!="try-error") {
+  if(!inherits(x1,"try-error")) {
     if(verbose) cat("\n\nThe following SaemixModel object was successfully created:\n\n")
     } else xmod<-"Creation of SaemixModel failed"
   if(verbose) print(xmod)
@@ -828,5 +1121,70 @@ mydiag <- function (x = 1, nrow, ncol) {
   y
 }
 
+############ VALIDITY OF COVARIANCE MODEL 
+
+
+#' Validate the structure of the covariance model
+#' 
+#' Check that a matrix corresponds to a structure defining a covariance model for a non-linear mixed effect model.
+#' Such a matrix should be composed of only 0s and 1s, with at least one element set to 1, and should be square and symmetrical.
+#' 1s on the diagonal indicate that the corresponding parameter has interindividual variability and that its variance will be estimated.
+#' 1s as off-diagonal elements indicate that a covariance between the two corresponding parameters will be estimated.
+#' 
+#' @param x	a matrix
+#' @param verbose	a boolean indicating whether warnings should be output if x is not a valid covariance model
+#' 
+#' @return a boolean, TRUE if x is an acceptable structure and FALSE if not. Messages will be output to describe why x isn't a valid covariance model if the argument verbose is TRUE.
+#' @seealso \code{SaemixModel}
+#' @author Emmanuelle Comets <emmanuelle.comets@@inserm.fr>, Belhal Karimi
+#' @keywords models
+#' @examples
+#' 
+#' covarmodel<-diag(c(1,1,0))
+#' validate.covariance.model(covarmodel) # should return TRUE
+#' 
+#' @export validate.covariance.model
+#' 
+validate.covariance.model <- function(x, verbose=TRUE){
+  #non-square matrix
+  if(dim(x)[1]!=dim(x)[2]) {
+    if(verbose) message("Error initialising SaemixModel object:\n   The covariance model needs to be a square matrix, please check dimensions.\n")
+    return(FALSE)
+  }
+  # only 0s
+  s <- sum(abs(x))
+  if(s==0) {
+    if(verbose) message("At least one parameter should have IIV in the model, the covariance model may not be only 0s.")
+#   return(FALSE)
+  }
+  
+  #values other than 1 or 0
+  s <- sum(x[x!=1 & x!=0])
+  if (s>0){
+    if(verbose) message("Error initialising SaemixModel object:\n  Invalid covariance model, only 0 or 1 values accepted, please change covariance model.\n")
+    return(FALSE)
+  }
+  
+  #asymmetrical
+  if (!all(t(x)==x)){
+    if(verbose) message("Error initialising SaemixModel object:\n  The matrix defining the covariance model is not symmetrical, please change covariance model.\n")
+    return(FALSE)
+  }
+  
+  #values other than 0 when diagonal number is 0
+  for (i in 1:nrow(x)){
+    for(j in 1:ncol(x)){
+      if (x[i,j]!=0){
+        if(x[i,i]==0 |x[j,j]==0){
+          if(verbose) message("Error initialising SaemixModel object:\n  The covariance model is invalid, please change covariance model.\n")
+          return(FALSE)
+        }
+      }
+    }
+  }
+  return(TRUE)
+}
+
+
+
 
-####################################################################################
