@@ -1,4 +1,4 @@
-  ####################################################################################
+####################################################################################
 ####			SaemixRes class - definition				####
 ####################################################################################
 
@@ -16,7 +16,8 @@
 #' 
 #' @section Objects from the Class: 
 #' An object of the SaemixData class can be created by using the function \code{\link{saemixData}} and contain the following slots:
-#' 
+#' @slot modeltype string giving the type of model used for analysis
+#' @slot status string indicating whether a model has been run successfully; set to "empty" at initialisation, used to pass on error messages or fit status
 #' @slot name.fixed a vector containing the names of the fixed parameters in the model
 #' @slot name.random a vector containing the names of the random parameters in the model
 #' @slot name.sigma a vector containing the names of the parameters of the residual error model
@@ -48,11 +49,11 @@
 #' @slot cond.mean.eta  a matrix giving the conditional mean estimates of the random effect eta
 #' @slot cond.shrinkage a vector giving the shrinkage on the conditional mean estimates of eta
 #' @slot mean.phi a matrix giving the population estimate (Ci*mu) including covariate effects, for each subject
-#' @slot map.psi a matrix giving the MAP estimates of individual parameters
-#' @slot map.phi a matrix giving the MAP estimates of individual phi
+#' @slot map.psi a dataframe giving the MAP estimates of individual parameters
+#' @slot map.phi a dataframe giving the MAP estimates of individual phi
 #' @slot map.eta a matrix giving the individual estimates of the random effects corresponding to the MAP estimates
 #' @slot map.shrinkage  a vector giving the shrinkage on the MAP estimates of eta
-#' @slot phi phi
+#' @slot phi individual parameters, estimated at the end of the estimation process as the average over the chains of the individual parameters sampled during the successive E-steps
 #' @slot psi.samp a three-dimensional array with samples of psi from the conditional distribution
 #' @slot phi.samp a three-dimensional array with samples of phi from the conditional distribution
 #' @slot phi.samp.var a three-dimensional array with the variance of phi
@@ -70,8 +71,8 @@
 #' @slot bic.gq Bayesian Information Criterion computed by Gaussian Quadrature
 #' @slot bic.covariate.gq Specific Bayesian Information Criterion for covariate selection computed by Gaussian Quadrature
 #' @slot predictions a data frame containing all the predictions and residuals in a table format
-#' @slot ypred a vector giving the mean population predictions obtained with the MAP estimates
-#' @slot ppred a vector giving the population predictions
+#' @slot ppred a vector giving the population predictions obtained with the population estimates
+#' @slot ypred a vector giving the mean population predictions
 #' @slot ipred a vector giving the individual predictions obtained with the MAP estimates
 #' @slot icpred a vector giving the individual predictions obtained with the conditional estimates
 #' @slot ires a vector giving the individual residuals obtained with the MAP estimates
@@ -92,11 +93,15 @@
 #'     \item{show}{\code{signature(object = "SaemixRes")}: prints details about the object}
 #'     \item{summary}{\code{signature(object = "SaemixRes")}: summary of the results. Returns a list with a number of elements extracted from the results ().}
 #' 	 }
-#' @references Comets  E, Lavenu A, Lavielle M. Parameter estimation in nonlinear mixed effect models using saemix, an R implementation of the SAEM algorithm. Journal of Statistical Software 80, 3 (2017), 1-41.
+#' @references E Comets, A Lavenu, M Lavielle M (2017). Parameter estimation in nonlinear mixed effect models using saemix,
+#' an R implementation of the SAEM algorithm. Journal of Statistical Software, 80(3):1-41.
 #' 
-#' Kuhn E, Lavielle M. Maximum likelihood estimation in nonlinear mixed effects models. Computational Statistics and Data Analysis 49, 4 (2005), 1020-1038.
+#' E Kuhn, M Lavielle (2005). Maximum likelihood estimation in nonlinear mixed effects models. 
+#' Computational Statistics and Data Analysis, 49(4):1020-1038.
+#' 
+#' E Comets, A Lavenu, M Lavielle (2011). SAEMIX, an R version of the SAEM algorithm. 20th meeting of the 
+#' Population Approach Group in Europe, Athens, Greece, Abstr 2173.
 #' 
-#' Comets E, Lavenu A, Lavielle M. SAEMIX, an R version of the SAEM algorithm. 20th meeting of the Population Approach Group in Europe, Athens, Greece (2011), Abstr 2173.
 #' @author Emmanuelle Comets \email{emmanuelle.comets@@inserm.fr}
 #' @author Audrey Lavenu
 #' @author Marc Lavielle.
@@ -112,6 +117,8 @@
 setClass(
   Class="SaemixRes",
   representation=representation(
+    modeltype="character", # string giving the type of the model used for analysis
+    status="character", # string indicating the status of the fit; set to "empty" at initialisation, used to pass on error messages or fit status
     name.fixed="character",	# names of fixed parameters in the model
     name.random="character",	# names of random effects
     name.sigma="character",	# names of parameters of residual error model
@@ -168,8 +175,8 @@ setClass(
     bic.covariate.gq="numeric",
 # Model predictions and residuals
 		predictions="data.frame", # data frame containing all the predictions and residuals below
-    ypred="numeric",		# vector of mean population predictions
-    ppred="numeric",		# vector of population predictions with MAP
+    ypred="numeric",        # vector of mean population predictions
+    ppred="numeric",        # vector of population predictions with the population parameters
     ipred="numeric",		# vector of individual predictions with MAP
     icpred="numeric",		# vector of individual predictions with conditional estimates
     ires="numeric",		  # vector of individual residuals with MAP (ipred-x)
@@ -189,7 +196,8 @@ setClass(
 # initialize
 
 #' @rdname initialize-methods
-#' 
+#' @param status string indicating whether a model has been run successfully; set to "empty" at initialisation, used to pass on error messages or fit status
+#' @param modeltype a character string  giving the model used for analysis
 #' @param name.fixed a character string  giving the name of the fixed parameters
 #' @param name.random a character string  giving the name of the random parameters
 #' @param fixed.effects vector with the estimates of h(mu) and betas in estimation order
@@ -212,8 +220,11 @@ setClass(
 setMethod(
   f="initialize",
   signature="SaemixRes",
-  definition= function(.Object,name.fixed,name.random,name.sigma,fixed.effects, fixed.psi,betaC,betas,omega,respar,cond.mean.phi,cond.var.phi,mean.phi,phi, phi.samp,parpop, allpar,MCOV){
+  definition= function(.Object,status="empty",modeltype,name.fixed,name.random,name.sigma,fixed.effects, fixed.psi,betaC,betas,omega,respar,cond.mean.phi,cond.var.phi,mean.phi,phi, phi.samp,parpop, allpar,MCOV){
 #    cat ("--- initialising SaemixRes Object --- \n")
+    .Object@status<-status
+    if(missing(modeltype)) modeltype<-character(0)
+    .Object@modeltype<-modeltype
     if(missing(name.fixed)) name.fixed<-character(0)
     .Object@name.fixed<-name.fixed
     if(missing(name.random)) name.random<-character(0)
@@ -263,7 +274,7 @@ setMethod(
 
 ##' Get/set methods for SaemixRes object
 ##' 
-##' Access slots of a SaemixRes object using the object["slot"] format
+##' Access slots of a SaemixRes object using the object\["slot"\] format
 ##' 
 #' @param x object
 #' @param i element to be replaced
@@ -281,6 +292,8 @@ setMethod(
   signature = "SaemixRes" ,
   definition = function (x,i,j,drop ){
   switch (EXPR=i,
+    "modeltype"={return(x@modeltype)},
+    "status"={return(x@status)},
     "name.fixed"={return(x@name.fixed)},
     "name.sigma"={return(x@name.sigma)},
     "name.random"={return(x@name.random)},
@@ -363,6 +376,8 @@ setReplaceMethod(
   signature = "SaemixRes" ,
   definition = function (x,i,j,value){
   switch (EXPR=i,
+    "modeltype"={x@modeltype<-value},
+    "status"={x@status<-value},
     "name.fixed"={x@name.fixed<-value},
     "name.random"={x@name.random<-value},
     "name.sigma"={x@name.sigma<-value},
@@ -448,18 +463,39 @@ setMethod("print","SaemixRes",
   function(x,digits=2,map=FALSE,...) {
 #    cat("Nonlinear mixed-effects model fit by the SAEM algorithm\n")
 #    cat("Dataset",x@name.data,"\n")
-    if(length(x@betas)==0) {
+    if(x@status %in% c("empty")) {
       cat("No fit performed yet.\n")
-      return("No fit performed")
+      return()
+    }
+    if(x@status %in% c("initial")) {
+      cat("No fit performed yet, parameters set by user.\n")
+      if(x@modeltype=="structural") {
+        tab<-cbind(c(x@name.fixed,x@name.sigma[x@indx.res]), c(x@fixed.effects,x@respar[x@indx.res]))
+      }else{
+        tab<-cbind(c(x@name.fixed), c(x@fixed.effects))
+      }
+      tab<-rbind(tab,
+                 cbind(x@name.random,diag(x@omega)[x@indx.omega]))
+      colnames(tab)<-c("Parameter","Value")
+      print(tab)
+      return()
     }
     cat("----------------------------------------------------\n")
     cat("-----------------  Fixed effects  ------------------\n")
     cat("----------------------------------------------------\n")
     if(length(x@se.fixed)==0) {
+      if(x@modeltype=="structural") {
       tab<-cbind(c(x@name.fixed,x@name.sigma[x@indx.res]), c(x@fixed.effects,x@respar[x@indx.res]))
+        }else{
+            tab<-cbind(c(x@name.fixed), c(x@fixed.effects))
+        }
       colnames(tab)<-c("Parameter","Estimate")
     } else {
+       if(x@modeltype=="structural") {
       tab<-cbind(c(x@name.fixed,x@name.sigma[x@indx.res]), c(x@fixed.effects,x@respar[x@indx.res]),c(x@se.fixed,x@se.respar[x@indx.res]))
+        }else{
+            tab<-cbind(c(x@name.fixed), c(x@fixed.effects),c(x@se.fixed))
+        }
       tab<-cbind(tab,100*abs(as.double(tab[,3])/as.double(tab[,2])))
       colnames(tab)<-c("Parameter","Estimate","SE","CV(%)")
       if(length(x@indx.cov)>0) {
@@ -492,6 +528,7 @@ setMethod("print","SaemixRes",
       tab<-cbind(tab,100*as.double(tab[,3])/as.double(tab[,2]))
       nampar<-unlist(strsplit(x@name.random,"omega2."))
       nampar<-nampar[nampar!=""]
+      if(length(nampar)>1) { # covariances only if more than one random effect in the model
       for(i in 1:(length(nampar)-1)) {
         for(j in (i+1):length(nampar)) {
           if(x@omega[i,j]!=0) {
@@ -500,6 +537,7 @@ setMethod("print","SaemixRes",
           }
         }
       }
+      }
       colnames(tab)<-c("Parameter","Estimate","SE","CV(%)")
     }
     if(digits>0) {
@@ -561,13 +599,39 @@ setMethod("print","SaemixRes",
 setMethod("show","SaemixRes",
   function(object) {
 #    cat("Nonlinear mixed-effects model fit by the SAEM algorithm\n")
+    if(object@status %in% c("empty")) {
+      cat("No fit performed yet.\n")
+      return()
+    }
+    if(object@status %in% c("initial")) {
+      cat("No fit performed yet, parameters set by user.\n")
+      if(object@modeltype=="structural") {
+        tab<-cbind(c(object@name.fixed,object@name.sigma[object@indx.res]), c(object@fixed.effects,object@respar[object@indx.res]))
+      }else{
+        tab<-cbind(c(object@name.fixed), c(object@fixed.effects))
+      }
+      tab<-rbind(tab,
+                 cbind(object@name.random,diag(object@omega)[object@indx.omega]))
+      colnames(tab)<-c("Parameter","Value")
+      print(tab)
+      return()
+    }
     cat("Fixed effects\n")
     if(length(object@se.fixed)==0) {
+      if(object@modeltype=="structural") {
       tab<-cbind(c(object@name.fixed,object@name.sigma[object@indx.res]), c(object@fixed.effects,object@respar[object@indx.res]))
+        }else{
+            tab<-cbind(c(object@name.fixed), c(object@fixed.effects))
+        }
       colnames(tab)<-c("Parameter","Estimate")
     } else {
-      tab<-cbind(c(object@name.fixed,object@name.sigma[object@indx.res]), c(object@fixed.effects,object@respar[object@indx.res]), c(object@se.fixed,object@se.respar[object@indx.res]))
+       if(object@modeltype=="structural") {
+            tab<-cbind(c(object@name.fixed,object@name.sigma[object@indx.res]), c(object@fixed.effects,object@respar[object@indx.res]),c(object@se.fixed,object@se.respar[object@indx.res]))
       tab<-cbind(tab,100*abs(as.double(tab[,3])/as.double(tab[,2])))
+        }else{
+            tab<-cbind(c(object@name.fixed), c(object@fixed.effects),c(object@se.fixed))
+            tab<-cbind(tab,100*abs(as.double(tab[,3])/as.double(tab[,3])))
+        }
       colnames(tab)<-c("Parameter","Estimate","  SE"," CV(%)")
       if(length(object@indx.cov)>0) {
       wstat<-as.double(tab[,2])/as.double(tab[,3])
@@ -588,7 +652,7 @@ setMethod("show","SaemixRes",
     print(tab,quote=FALSE)
 
     cat("\nVariance of random effects\n")
-#  cat("   ECO TODO: check if Omega or Omega2 (SD or variances) and can we choose ?\n")
+#  cat("   ECO TODO: check if Omega or Omega2 (SD orxres1@ezrzr variances) and can we choose ?\n")
     if(length(object@se.omega)==0) {
       tab<-cbind(object@name.random,diag(object@omega)[object@indx.omega])
       colnames(tab)<-c("Parameter","Estimate")
@@ -597,6 +661,7 @@ setMethod("show","SaemixRes",
       tab<-cbind(tab,100*as.double(tab[,3])/as.double(tab[,2]))
       nampar<-unlist(strsplit(object@name.random,"omega2."))
       nampar<-nampar[nampar!=""]
+      if(length(nampar)>1) { # covariances only if more than one random effect in the model
       for(i in 1:(length(nampar)-1)) {
         for(j in (i+1):length(nampar)) {
           if(object@omega[i,j]!=0) {
@@ -605,17 +670,15 @@ setMethod("show","SaemixRes",
           }
         }
       }
+      }
       colnames(tab)<-c("Parameter","Estimate","  SE"," CV(%)")
     }
       for(i in 2:dim(tab)[2]) 
          tab[,i]<-format(as.double(as.character(tab[,i])),digits=3)
     rownames(tab)<-rep("",dim(tab)[1])
     print(tab,quote=FALSE)
-    if(length(object@ll.lin)>0 | length(object@ll.is)>0 | length(object@ll.gq)>0) {
-    cat("\nStatistical criteria\n")
-    }
     mat1<-object@omega
-    if(sum(abs(mat1-diag(diag(mat1))))>0) {
+    if(sum(abs(mat1-mydiag(mydiag(mat1))))>0) {
     cat("\nCorrelation matrix of random effects\n")
     tab<-cov2cor(object@omega[object@indx.omega,object@indx.omega,drop=FALSE])
     for(i in 1:dim(tab)[2]) 
@@ -623,6 +686,9 @@ setMethod("show","SaemixRes",
     try(colnames(tab)<-rownames(tab)<-object@name.random)
     print(tab,quote=FALSE)
     }
+    if(length(object@ll.lin)>0 | length(object@ll.is)>0 | length(object@ll.gq)>0) {
+      cat("\nStatistical criteria\n")
+    }
     if(length(object@ll.lin)>0) {
     cat("Likelihood computed by linearisation\n")
     cat("      -2LL=",(-2*object@ll.lin),"\n")
@@ -653,15 +719,53 @@ setMethod("show","SaemixRes",
 # Could be print, with only head of data
 setMethod("showall","SaemixRes",
   function(object) {
+    if(object@status %in% c("empty")) {
+      cat("No fit performed yet.\n")
+      return()
+    }
+    if(object@status %in% c("initial")) {
+      cat("No fit performed yet, parameters set by user.\n")
+      if(object@modeltype=="structural") {
+        tab<-cbind(c(object@name.fixed,object@name.sigma[object@indx.res]), c(object@fixed.effects,object@respar[object@indx.res]))
+      }else{
+        tab<-cbind(c(object@name.fixed), c(object@fixed.effects))
+      }
+      tab<-rbind(tab,
+                 cbind(object@name.random,diag(object@omega)[object@indx.omega]))
+      colnames(tab)<-c("Parameter","Value")
+      print(tab)
+      return()
+    }
     cat("\n----------------------------------------------------\n")
     cat("-----------------  Fixed effects  ------------------\n")
     cat("----------------------------------------------------\n")
     if(length(object@se.fixed)==0) {
+      if(object@modeltype=="structural") {
       tab<-cbind(c(object@name.fixed,object@name.sigma[object@indx.res]), c(object@fixed.effects,object@respar[object@indx.res]))
+        }else{
+            tab<-cbind(c(object@name.fixed), c(object@fixed.effects))
+        }
       colnames(tab)<-c("Parameter","Estimate")
     } else {
-      tab<-cbind(c(object@name.fixed,object@name.sigma[object@indx.res]), c(object@fixed.effects,object@respar[object@indx.res]), c(object@se.fixed,object@se.respar[object@indx.res]))
+       if(object@modeltype=="structural") {
+            tab<-cbind(c(object@name.fixed,object@name.sigma[object@indx.res]), c(object@fixed.effects,object@respar[object@indx.res]),c(object@se.fixed,object@se.respar[object@indx.res]))
       tab<-cbind(tab,100*abs(as.double(tab[,3])/as.double(tab[,2])))
+            nampar<-unlist(strsplit(object@name.random,"omega2."))
+            nampar<-nampar[nampar!=""]
+            if(length(nampar)>1) { # covariances only if more than one random effect in the model
+              for(i in 1:(length(nampar)-1)) {
+                for(j in (i+1):length(nampar)) {
+                  if(object@omega[i,j]!=0) {
+                    covar<-c(paste("cov",nampar[i],nampar[j],sep="."),object@omega[i,j],object@se.cov[i,j],object@se.cov[i,j]/object@omega[i,j]*100)
+                    tab<-rbind(tab,covar)
+                  }
+                }
+              }
+            }
+        } else{
+            tab<-cbind(c(object@name.fixed), c(object@fixed.effects),c(object@se.fixed))
+            tab<-cbind(tab,100*abs(as.double(tab[,2])/as.double(tab[,1])))
+        }
       colnames(tab)<-c("Parameter","Estimate","SE","CV(%)")
       if(length(object@indx.cov)>0) {
       wstat<-as.double(tab[,2])/as.double(tab[,3])
@@ -769,7 +873,7 @@ setMethod("showall","SaemixRes",
 resid.SaemixRes<-function (object, type = c("ires", "wres", "npde", "pd", "iwres", "icwres"), ...) {
             type <- match.arg(type)
             res <- switch(type, ires=object@ires, wres=object@wres, npde=object@npde, iwres=object@iwres, icwres=object@icwres, pd=object@pd)
-            res
+            if(length(res)>0) return(res) else message(paste("No residuals of type",type,"available\n"))
 }
 
 
@@ -781,7 +885,7 @@ resid.SaemixRes<-function (object, type = c("ires", "wres", "npde", "pd", "iwres
 #' @aliases fitted fitted.SaemixRes
 #' 
 #' @param object an object of type SaemixRes or SaemixObject
-#' @param type string determining which predictions are extracted. Possible values are: "ipred" (individual predictions obtained using the mode of the individual distribution for each subject, default), "ypred" (population predictions obtained using the population parameters f(E(theta))), "ppred" (mean of the population predictions (E(f(theta)))) and "icpred" (individual predictions obtained using the conditional mean of the individual distribution). See user guide for details.
+#' @param type string determining which predictions are extracted. Possible values are: "ipred" (individual predictions obtained using the mode of the individual distribution for each subject, default), "ppred" (population predictions obtained using the population parameters f(E(theta))), "ypred" (mean of the population predictions (E(f(theta)))) and "icpred" (individual predictions obtained using the conditional mean of the individual distribution). See user guide for details.
 #' @param ... further arguments to be passed to or from other methods
 #' 
 #' @return Model predictions
@@ -792,12 +896,12 @@ resid.SaemixRes<-function (object, type = c("ires", "wres", "npde", "pd", "iwres
 
 fitted.SaemixRes<-function (object, type = c("ipred", "ypred", "ppred", "icpred"), ...) {
   type <- match.arg(type)
-  pred <- switch(type, ipred=object@ipred, ypred=object@ypred, ppred=object@ppred, ipred=object@ipred, icpred=object@icpred)
-  pred
+  pred <- switch(type, ipred=object@ipred, ypred=object@ypred, ppred=object@ppred, icpred=object@icpred)
+  if(length(pred)>0) return(pred) else message(paste("No fitted values of type",type,"available\n"))
 }
 
 ####################################################################################
-####			SaemixRes class - variance-covariance matrix		####
+####            SaemixRes class - extract variance-covariance matrix            ####
 ####################################################################################
 
 #' Extracts the Variance-Covariance Matrix for a Fitted Model Object
@@ -805,7 +909,7 @@ fitted.SaemixRes<-function (object, type = c("ipred", "ypred", "ppred", "icpred"
 #' Returns the variance-covariance matrix of the main parameters of a fitted model object
 #' 
 #' @name vcov
-#' @aliases vcov vcov.SaemixRes vcov.SaemixObject ##vcov,SaemixRes vcov,SaemixObject
+#' @aliases vcov vcov.SaemixRes vcov.SaemixObject
 #' 
 #' @param object a fitted object from a call to saemix
 #' @param ...	further arguments to be passed to or from other methods
@@ -815,7 +919,8 @@ fitted.SaemixRes<-function (object, type = c("ipred", "ypred", "ppred", "icpred"
 #' @export
 
 vcov.SaemixRes<-function(object, ...) {
-  try(solve(object@fim))
+  if(object@status!="empty" && min(dim(object@fim))>0)
+      try(solve(object@fim)) else NULL
 }
 
 #' @rdname vcov
@@ -828,7 +933,7 @@ vcov.SaemixObject<-function(object, ...) {
 # setMethod("vcov","SaemixRes",
 #           function (object, ...) 
 #           {
-#             object@fim
+#             try(solve(object@fim))
 #           }
 # )
 # 
@@ -844,6 +949,7 @@ vcov.SaemixObject<-function(object, ...) {
 ####			SaemixRes class - method to plot			####
 ####################################################################################
 
+
 ####################################################################################
 ####				SaemixRes class - summary method			####
 ####################################################################################
@@ -865,10 +971,18 @@ setMethod("summary","SaemixRes",
             }
             #    browser()
             if(length(object@se.fixed)==0) {
+              if(object@modeltype=="structural") {
                 tab<-data.frame(c(object@name.fixed, object@name.sigma[object@indx.res]), c(object@fixed.effects,object@respar[object@indx.res]))
+              }else{
+                tab<-data.frame(c(object@name.fixed), c(object@fixed.effects))
+              }
               colnames(tab)<-c("Parameter","Estimate")
             } else {
+              if(object@modeltype=="structural") {
                 tab<-data.frame(c(object@name.fixed, object@name.sigma[object@indx.res]), c(object@fixed.effects,object@respar[object@indx.res]),c(object@se.fixed,object@se.respar[object@indx.res]), stringsAsFactors=FALSE)
+              }else{
+                tab<-data.frame(c(object@name.fixed), c(object@fixed.effects),c(object@se.fixed), stringsAsFactors=FALSE)
+              }
               tab<-cbind(tab,100*abs(as.double(tab[,3])/as.double(tab[,2])))
               colnames(tab)<-c("Parameter","Estimate","SE","CV(%)")
               if(length(object@indx.cov)>0) {
@@ -974,7 +1088,7 @@ setMethod("summary","SaemixRes",
             res<-list(modeltype=object@modeltype,fixed.effects=tab.fix,sigma=sigma,random.effects=tab.random, correlation.matrix=tab.corr,logLik=tab.ll,coefficients=coef)
             if(length(object@fim)>0) res$FIM<-object@fim
             if(length(object@ypred)>0 | length(object@ipred)>0  | length(object@ppred)>0 | length(object@icpred)>0) {
-              res$fitted<-list(population=list(pop.param=object@ppred, pop.mean=object@ypred),individual=list(map.ipred=object@ipred, cond.ipred=object@icpred))
+              res$fitted<-list(population=list(pop.param=object@ppred, pop.mean=object@ppred),individual=list(map.ipred=object@ipred, cond.ipred=object@icpred))
             }
             if(length(object@wres)>0 | length(object@iwres)>0  | length(object@icwres)>0 | length(object@pd)>0) {
               res$residuals<-list(population=list(wres=object@wres), individual=list(map.iwres=object@iwres,cond.iwres=object@icwres, pd=object@pd, npde=object@npde))
@@ -985,4 +1099,3 @@ setMethod("summary","SaemixRes",
 )
 
 ####################################################################################
-####################################################################################
